
//const Tweener = imports.ui.tweener;
const Shell = imports.gi.Shell;
const St = imports.gi.St;
const Main = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;
const Search = imports.ui.search;
const SearchDisplay = imports.ui.searchDisplay;
const IconGrid = imports.ui.iconGrid;
const Gio = imports.gi.Gio;
const Meta = imports.gi.Meta;
const Util = imports.misc.util;
const Config = imports.misc.config;
const Gtk = imports.gi.Gtk;
const Lang = imports.lang;
const GLib = imports.gi.GLib;

const Clutter = imports.gi.Clutter;
const GnomeDesktop = imports.gi.GnomeDesktop;
const Pango = imports.gi.Pango;
const Signals = imports.signals;
const Mainloop = imports.mainloop;
const Gettext = imports.gettext.domain('gnome-shell');
const _ = Gettext.gettext;
const C_ = Gettext.pgettext;
const Tp = imports.gi.TelepathyGLib;

const TEXT = "Texto";
const SOUND = "Som";
const PICTURE = "Imagem";
const VIDEO = "V\xEDdeo";
const LESSON = "Aula";

let text, button;
let educSearch;
let educInstall;
let searchDisplay;
let viewTab;
let tabIndex;

const Me = imports.misc.extensionUtils.getCurrentExtension();
const DocInfo = Me.imports.docInfo;

const MAX_PAGES = 20;
const MAX_SEARCH_RESULTS_ROWS = 10;
const MAX_RESULTS_PER_PAGE = 24;
const ITEM_LIMIT = 24;
const ICON_SIZE = 40;
const SEARCH_XML = '/var/lib/le-search/search.xml'
const RESULT_DIR = GLib.get_home_dir() + '/.le-search'

GLib.spawn_command_line_sync('mkdir -p ' + RESULT_DIR);

/* ------------------------------------------------------------------------------------------------- */

function SearchResult(result)
{
    this._init(result);
};

SearchResult.prototype =
{
    _init: function(resultMeta)
    {

        this.actor = new St.Bin({ style_class: 'contact',
                                 reactive: true,
                                 can_focus: true,
                                 track_hover: true });

        let content = new St.BoxLayout( { style_class: 'contact-content',
            vertical: false});
        this.actor.set_child(content);

        let iconObj = Gio.app_info_get_default_for_type(resultMeta.type[0],null).get_icon();
        let icon = St.TextureCache.get_default().load_gicon(null, iconObj, ICON_SIZE);

        content.add(icon, { x_fill: true,
                            y_fill: false,
                            x_align: St.Align.START,
                            y_align: St.Align.MIDDLE });

        let result = new St.BoxLayout({ style_class: 'contact-details',
                                        vertical: true});

        content.add(result, { x_fill: false, y_fill: false, x_align: St.Align.START });
        //this.content.set_child(result);

        //this.actor.connect('clicked', Lang.bind(this, this.onClickedContent));

        let authorLabel = new St.Label({ text: resultMeta.author,
                                             style_class: 'author'});
        let titleLabel = new St.Label({ text: resultMeta.title,
                                             style_class: 'title' });
        let categoryLabel = new St.Label({ text: resultMeta.category,
                                             style_class: 'category' });
        /*let debugLabel = new St.Label({ text: resultMeta.type[0],
                                             style_class: 'debug' });*/
        result.add(titleLabel, { x_fill: false, x_align: St.Align.START });
        result.add(authorLabel, { x_fill: false, x_align: St.Align.START });
        result.add(categoryLabel, { x_fill: false, x_align: St.Align.START });
        //result.add(debugLabel, { x_fill: false, x_align: St.Align.START });

    }


};

/* ------------------------------------------------------------------------------------------------- */

function searchContent(terms, contents, sType, win)
{

    let xmlFile;
    let filters = terms.join(' ');
    GLib.spawn_command_line_sync('parses-contents ' +
        SEARCH_XML + ' ' + RESULT_DIR + '/result-' + sType + '.xml ' + sType + ' "' +
        filters + '" ' + filters);
    xmlFile = Shell.get_file_contents_utf8_sync(RESULT_DIR + '/result-' + sType + '.xml').replace(/^<\?xml version="1.0" encoding="UTF-8"\?>/, '');

    let results = new XML(xmlFile);

    let start = 0;
    let end = results.result.length();
    if(sType != 'install' && sType != 'search')
    {
        start = win*MAX_RESULTS_PER_PAGE;
        if(start+MAX_RESULTS_PER_PAGE < end)
            end = start+MAX_RESULTS_PER_PAGE;
    }

    for( i=start ; i < end ; i++)
    {
        contents.push(
        {
            'title'     : results.result[i].@title.toString(),
            'author'    : results.result[i].@author.toString(),
            'category' : results.result[i].@category.toString(),
            'path' : results.result[i].@path.toString(),
            'package' : results.result[i].@package.toString(),
            'type' : Gio.content_type_guess(results.result[i].@path.toString(), null)
        });
        //if(contents.length > 19) return;
    }

    return results.result.length();
}

function foundContent(content, results, term)
{
    for(var i in results)
    {
        if( results[i].title == content.@title &&
            results[i].author == content.@author &&
            results[i].category == content.@category )
            return;
    }
    if( content.@title.toString().toLowerCase().search(term) != -1 ||
        content.@author.toString().toLowerCase().search(term) != -1 ||
        content.@category.toString().toLowerCase().search(term) != -1 )
    {
        return;
    }
}

/* ------------------------------------------------------------------------------------------------- */

function EducSearch()
{
    this._init.apply(this, arguments);
};

EducSearch.prototype =
{
    __proto__: Search.SearchProvider.prototype,

    _init: function(title, install)
    {
        Search.SearchProvider.prototype._init.call(this, title);
        this.install = install;
    },


    getInitialResultSet: function(terms) {
        //let expr = terms.join('');
        let result = [];

        if(this.install)
            searchContent(terms, result, 'install', 0);
        else
            searchContent(terms, result, 'search', 0);

        return result;
    },

    getSubsearchResultSet: function(prevResults, terms) {
        return this.getInitialResultSet(terms);
    },

    getResultMetas: function(ids) {
        let metas = [];
        for (let i = 0; i < ids.length; i++) {
            metas.push({ 'id': ids[i],
                         'name': ids[i].title,
                         'createIcon': function(size)
                                        {
                                            let iconObj = Gio.app_info_get_default_for_type(ids.type[0],null).get_icon();
                                            return St.TextureCache.get_default().load_gicon(null, iconObj, ICON_SIZE);
                                        } });
        }
        return metas;
    },

    activateResult: function(id)
    {
        if(this.install)
        {
            Util.spawn(['le-install', id.package]);
        }
        else
        {
            Gio.app_info_launch_default_for_uri('file://' + id.path, global.create_app_launch_context());
        }
    },

    createResultActor: function(resultMeta, terms) {
        let result = new SearchResult(resultMeta.id);
        return result.actor;
    },

    createResultContainerActor: function() {
        /*let grid = new IconGrid.IconGrid({ rowLimit: MAX_SEARCH_RESULTS_ROWS,
                                           xAlign: St.Align.START });*/
        let grid = new IconGrid.IconGrid({ xAlign: St.Align.START });
        grid.actor.style_class = 'contact-grid';

        let actor = new SearchDisplay.GridSearchResults(this, grid);
        return actor;
    },
}

/* ------------------------------------------------------------------------------------------------- */

function EventItem (result, multi_select, journal_layout) {
    this._init (result, multi_select, journal_layout);
}

EventItem.prototype = {
    _init: function (result, multi_select, journal_layout) {
        if (!result)
            throw new Error ("result must not be null");

        this.result = result;

        this._journal_layout = journal_layout;
        this._item_info = new DocInfo.ZeitgeistItemInfo (result);

        this.actor = new St.Bin({ style_class: 'contact',
                                 reactive: true,
                                 can_focus: true,
                                 track_hover: true });

        this.content = new St.BoxLayout( { style_class: 'contact-content',
            vertical: false});

        /* octect-stream type behave as a pdf but the following function does not recognize it */
        let type = result.type[0] == 'application/octet-stream'? 'application/pdf' : result.type[0];
	
        let iconObj = Gio.app_info_get_default_for_type(type,null);

       	if (iconObj) {
	        iconObj = iconObj.get_icon();
                this.icon = St.TextureCache.get_default().load_gicon(null, iconObj, ICON_SIZE);
                this.content.add(this.icon, { x_fill: true,
                                y_fill: false,
                                x_align: St.Align.START,
                                y_align: St.Align.MIDDLE });
        }

        this.details = new St.BoxLayout({ style_class: 'contact-details',
                                        vertical: true});

        this.content.add(this.details, { x_fill: false, y_fill: false, x_align: St.Align.START });

        let authorLabel = new St.Label({ text: result.author,
                                             style_class: 'author'});
        let titleLabel = new St.Label({ text: result.title,
                                             style_class: 'title' });
        let categoryLabel = new St.Label({ text: result.category,
                                             style_class: 'category' });
        /*let debugLabel = new St.Label({ text: this._item_info.type[0],
                                             style_class: 'debug' });*/
        this.details.add(titleLabel, { x_fill: false, x_align: St.Align.START });
        this.details.add(authorLabel, { x_fill: false, x_align: St.Align.START });
        this.details.add(categoryLabel, { x_fill: false, x_align: St.Align.START });
        //result.add(debugLabel, { x_fill: false, x_align: St.Align.START });

        //this.actor.connect('enter-event', Lang.bind(this, this._onEnter));
        //this.actor.connect('leave-event', Lang.bind(this, this._onLeave)); 

        this._button = new St.Button ({ reactive: true,
                                        can_focus: true,
                                        button_mask: St.ButtonMask.ONE | St.ButtonMask.THREE, // assume button 2 (middle) does nothing
                                        x_fill: true,
                                        y_fill: true });
        this._button.set_child (this.content);
        this._button.connect ('clicked', Lang.bind(this, this._onClicked));

        /*this._closeButton = new St.Button ({ style_class: "window-close" });
        this._closeButton.connect ('clicked', Lang.bind(this, this._removeItem));
        this._closeButton.connect ('style-changed',
                                         Lang.bind(this, this._onStyleChanged));*/
    
        //Util.spawn(['imprime_debug', this.result.author]); // DEBUG: #################################
        this.actor.set_child (this._button);
        //this.content.add(this._closeButton);

        //this._closeButton.hide();

        this._idleToggleCloseId = 0;
        this._menuTimeoutId = 0;
        this._menuDown = 0;

        this._menu = null;
        this._menuManager = new PopupMenu.PopupMenuManager(this);

        this.multiSelect = multi_select;
    },

    _onDestroy: function() {
        //if (this._journalLayout != undefined)
        //    this._journalLayout.removeHSpace(this);
    },

    _removeMenuTimeout: function() {
        if (this._menuTimeoutId > 0) {
            Mainloop.source_remove(this._menuTimeoutId);
            this._menuTimeoutId = 0;
        }
    },

    // callback for this._button's "clicked" signal
    _onClicked: function (actor, button) {
        this._removeMenuTimeout();
        if (button == 1) {
            let modifiers = Clutter.get_current_event().get_state();
            if (modifiers & Clutter.ModifierType.CONTROL_MASK) {
                this.multiSelect.select (this._button, this._item_info);
            } else {
                let elements = this.multiSelect.querySelections ();
                this._launchAll(elements);
            }
        } else if (button == 3) {
            this._popupMenu();
            this._idleToggleCloseButton ();
        }
        return true;
    },

    _launchAll: function(elements) {
        if (elements.length > 1) {
            for (let i = 0; i < elements.length; i++) {
                let e = elements[i];
                if (e.item.subject.interpretation == Semantic.NMO_IMMESSAGE)
                    Util.spawn(['empathy', e.item.subject.uri]);
                else
                    e.item.launch ();
            }
            this.multiSelect.destroy ();
        } else {
            this._item_info.launch ();
            Main.overview.hide ();
        }
    },

    _onEnter: function() {
        this._closeButton.show();
    },
    
    _onLeave: function() {
        if (this._idleToggleCloseId == 0)
            this._idleToggleCloseId = Mainloop.timeout_add(10, Lang.bind(this, this._idleToggleCloseButton));
    },

    _idleToggleCloseButton: function() {
        this._idleToggleCloseId = 0;
        if ((!this._button.has_pointer &&
              !this._closeButton.has_pointer) ||
              this._menu)
            this._closeButton.hide();

        return false;
    },

    // FIXME: Calculate (X) positions.
    _updatePosition: function () {
        let closeNode = this._closeButton.get_theme_node();
        this._closeButton._overlap = closeNode.get_length('-shell-close-overlap');

        let [buttonX, buttonY] = this._button.get_position();
        let [buttonWidth, buttonHeight] = this._button.get_size();
        
        buttonWidth = buttonWidth - 14 //this.actor.scale_x * (buttonWidth - 16);
        buttonHeight = buttonHeight - 12 //this.actor.scale_y * (buttonHeight - 16);
        
        //this._closeButton.y = buttonY - (this._closeButton.height - this._closeButton._overlap);
        this._closeButton.x = buttonX + (buttonWidth - this._closeButton._overlap);
    },

    _removeItem: function () {
        this.actor.connect('destroy', Lang.bind(this, this._onDestroy)); 
        _deleteEvents(this._item_info.name);
        this.multiSelect.unselect (this._button, this._item_info);
        this._journal_layout.removeItem(this);
        this._journal_layout.refresh();
        this.actor.destroy();
    },

    destroy: function (source, item) {
    },

    _onStyleChanged: function () {
        this._updatePosition ();
        this._closeButton.set_position (Math.floor(this._closeButton.x), Math.floor(this._closeButton.y));
    },

    _disconnectSignals: function() {
        this._menu.close();
        Main.overview.disconnect(this._overviewHidingId);
    },

    _popupMenu: function() {
        this._removeMenuTimeout();
        this._button.fake_release();
        if (!this._menu) {
            this._menu = new ActivityIconMenu(this);
            this._menu.connect('activate-window', Lang.bind(this, function (menu, window) {
                this.activateWindow(window);
            }));
            this._menu.connect('popup', Lang.bind(this, function (menu, isPoppedUp) {
                if (!isPoppedUp)
                    this._onMenuPoppedDown();
            }));
            
            this._overviewHidingId = Main.overview.connect('hiding', Lang.bind(this, function () { this._disconnectSignals();}));

            this._menuManager.addMenu(this._menu);
        }

        this._button.set_hover(true);
        this._button.show_tooltip();
        this._menu.popup();

        return false;
    },
    
    activateWindow: function(metaWindow) {
        if (metaWindow) {
            Main.activateWindow(metaWindow);
        } else {
            Main.overview.hide();
        }
    },

    _onMenuPoppedDown: function() {
        this._button.sync_hover();
    }

};

function HeadingItem (label_text) {
    this._init (label_text);
}

HeadingItem.prototype = {
    _init: function (label_text) {
        this._label_text = label_text;
        this.actor = new St.Label ({ text: this._label_text.toUpperCase(),
            style_class: 'journal-heading',});
    }
};

function ActivityIconMenu(source) {
    this._init(source);
}

ActivityIconMenu.prototype = {
    __proto__: PopupMenu.PopupMenu.prototype,

    _init: function(source) {
        let side = St.Side.LEFT;
        if (St.Widget.get_default_direction() == St.TextDirection.RTL)
            side = St.Side.RIGHT;

        PopupMenu.PopupMenu.prototype._init.call(this, source.actor, 0.5, side, 0);

        // We want to keep the item hovered while the menu is up
        this.blockSourceEvents = true;

        this._source = source;
        //this._favs = new FavoriteItem ();
        this._item = this._source._item_info;
       
        this._openWith = [];
        this._openWithItem = [];

        this.connect('activate', Lang.bind(this, this._onActivate));
        this.connect('open-state-changed', Lang.bind(this, this._onOpenStateChanged));

        this.actor.add_style_class_name('app-well-menu');

        // Chain our visibility and lifecycle to that of the source
        source.actor.connect('notify::mapped', Lang.bind(this, function () {
            if (!source.actor.mapped)
                this.close();
        }));
        source.actor.connect('destroy', Lang.bind(this, function () { this.actor.destroy(); }));

        Main.uiGroup.add_actor(this.actor);
    },

    _redisplay: function() {
        this.removeAll();
        let selections = this._source.multiSelect.querySelections ();
        
        if (selections.length < 2 || this._source.multiSelect.isSelected(this._item)) {
            if (this._item.subject.interpretation == Semantic.NMO_IMMESSAGE) {
                this._startConversation = this._appendMenuItem(_("Start a new conversation"));
                // dead menu entry, will probably be replaced by an option to launch gnome-contacts 
                // http://blogs.gnome.org/alexl/2011/06/13/announcing-gnome-contacts/
                this._previousConversations = this._appendMenuItem(_("Previous conversations"));
                this._appendSeparator();
            }
            else {
                let apps = Gio.app_info_get_recommended_for_type(this._item.subject.mimetype); 
                if (apps.length > 0) {      
                    for (let i = 0; i < apps.length; i++) {
                        this._openWith.push(this._appendMenuItem(_("Open with " + apps[i].get_name())));
                        this._openWithItem.push(apps[i]);
                    }
                    this._appendSeparator();
                }
            }
            //let isFavorite = this._favs.isFavorite(this._item.subject.uri);
            //this._toggleFavoriteMenuItem = this._appendMenuItem(isFavorite ? _("Remove from Favorites")
            //                                                         : _("Add to Favorites"));
            this._showItemInManager = this._appendMenuItem(_("Show in file manager"));
            this._moveFileToTrash = this._appendMenuItem(_("Move to trash"));
        } else {
            this._launchAllItems = this._appendMenuItem(_("Launch items"));
            this._appendSeparator();
            this._showItemsInManager = this._appendMenuItem(_("Show items file manager"));
            this._moveFilesToTrash = this._appendMenuItem(_("Move items to trash"));
        }
    },

    _appendSeparator: function () {
        let separator = new PopupMenu.PopupSeparatorMenuItem();
        this.addMenuItem(separator);
    },

    _appendMenuItem: function(labelText) {
        // FIXME: app-well-menu-item style
        let item = new PopupMenu.PopupMenuItem(labelText);
        this.addMenuItem(item);
        return item;
    },

    popup: function(activatingButton) {
        this._redisplay();
        this.open();
    },

    _onOpenStateChanged: function (menu, open) {
        if (open) {
            this.emit('popup', true);
        } else {
            this.emit('popup', false);
        }
    },

    _onActivate: function (actor, child) {
        let selections = this._source.multiSelect.querySelections ();
        let menuIndex = this._openWith.indexOf(child);
        if (child._window) {
            let metaWindow = child._window;
            this.emit('activate-window', metaWindow);
        } else if (menuIndex > -1) {
              this._openWithItem[menuIndex].launch_uris([this._item.subject.uri], null);
        } else if (child == this._launchAllItems) {
            this._source._launchAll(selections);
        /*} else if (child == this._toggleFavoriteMenuItem) {
            /*
            let isFavorite = this._favs.isFavorite(this._item.subject.uri);
            if (isFavorite) {
                this._favs.deleteBookmarkWithUri (this._item.subject.uri);
                this._source.actor.destroy();
            } else {
                this._favs.append (this._item.subject.uri);
            }
            */
        } else if (child == this._showItemInManager) {
            Util.spawn(['nautilus', this._item.subject.uri]);
            Main.overview.hide();
        } else if (child == this._showItemsInManager) {
            this._launchItemsInManager(selections);
        } else if (child == this._moveFileToTrash) {
            // remove the item from journal after trashing, it'll be recuperated
            // as a new event by the Trash filter
            let uri = this._item.subject.uri;
            try {
              let file = Gio.file_new_for_uri(uri);
              file.trash(null);
            } catch(e) {
              Util.spawn(['gvfs-trash', uri]);
            }
            this._source._removeItem();
        } else if (child == this._moveFilesToTrash) {
            this._moveItemsToTrash(selections);
        } else if (child == this._startConversation) {
            this._telepathyConversationLaunch(this._item.subject.origin, this._item.subject.uri);
            //Util.spawn(['empathy', this._item.subject.uri]);
        }
        this.close();
        this._source.multiSelect.destroy ();
    },


    _telepathyConversationLaunch: function(account_id, contact_id) {
        let props = {};
        //let dbus = Tp.DBusDaemon.dup();
        let account_manager = new Tp.AccountManager.dup();

        let account = account_manager.ensure_account(account_id);
        
        props[Tp.PROP_CHANNEL_CHANNEL_TYPE] = Tp.IFACE_CHANNEL_TYPE_TEXT;
        props[Tp.PROP_CHANNEL_TARGET_HANDLE_TYPE] = Tp.HandleType.CONTACT; 
        props[Tp.PROP_CHANNEL_TARGET_ID] = contact_id; 

        let req = new Tp.AccountChannelRequest(account, props, global.get_current_time());

        req.ensure_channel_async('', null, null);
    }, 

    // TOO REDUNDANT FIXME

    _launchItemsInManager: function(elements) {
        if (elements.length > 1) {
            for (let i = 0; i < elements.length; i++) {
                let e = elements[i];
                Util.spawn(['nautilus', e.item.subject.origin]);
            }
        } else {
            Util.spawn(['nautilus', this._item.subject.origin]);
            Main.overview.hide ();
        }
    },

    _moveItemsToTrash: function(elements) {
        if (elements.length > 1) {
            for (let i = 0; i < elements.length; i++) {
                let e = elements[i];
                let uri = e.item.subject.uri;
                try {
                  let file = Gio.file_new_for_uri(uri);
                  file.trash(null);
                } catch(e) {
                  Util.spawn(['gvfs-trash', uri]);
                }
                this._source._removeItem();
            }
        }
    }

};



let SubjCategories = {};

function SubEduc (label, sorting, multi_select, subFilter, filter, expandable) {
    this._init (label, sorting, multi_select, subFilter, filter, expandable);
}

SubEduc.prototype = {
    _init: function (label, sorting, multi_select, subFilter, filter, expandable) {
        this._expandable = expandable;
        this._item_limit = ITEM_LIMIT;
        //this._subjects = subjects; //*
        this._items = [];
        this._events = []; //*
        this._results = [];
        this._filter = filter;
        this._subFilter = subFilter;
        //this._timerange = timerange; //*
        //this._template = template; //*
        this._sorting = sorting;
        this._multi_select = multi_select;
        this._itemSpacing = 37; // "item-spacing" attribute
        this._rowSpacing = 8;  // "row-spacing" attribute11
        this._box = new St.BoxLayout({vertical: true });
        this._container = new Shell.GenericContainer ();
        this._container.connect ("style-changed", Lang.bind (this, this._styleChanged));
        this._container.connect ("allocate", Lang.bind (this, this._allocate));
        this._container.connect ("get-preferred-width", Lang.bind (this, this._getPreferredWidth));
        this._container.connect ("get-preferred-height", Lang.bind (this, this._getPreferredHeight));
        this._header = new St.BoxLayout({ vertical: false, style_class: 'journal-heading-box'});
        this._box.add_actor (this._header, { x_align: St.Align.START, y_align: St.Align.START, expand: true, x_fill: true, y_fill: false });
        this._box.add_actor (this._container, { expand: true, x_fill: true, y_fill: true});
        this.actor = this._box;
        this._label = label;
        this._inserted_items = [];
        this._expanded = false;
        this._page = 0;

        this.refresh();
    },


    refresh : function () {
        this._items = []

        var widgets = this._container.get_children();
        for (var i = 0; i < widgets.length; i++)
        {
            this._container.remove_actor(widgets[i]);
        }
        var widgets = this._header.get_children();
        for (var i = 0; i < widgets.length; i++)
        {
            this._header.remove_actor(widgets[i]);
        }
        
        var heading = new HeadingItem(this._label);
        
        //this.appendItem (heading);
        
        this._header.add_actor(heading.actor, {x_align: St.Align.START, y_align: St.Align.START, expand: false, x_fill: true, y_fill: true });

        this._moreButton = new St.Button({
            label: GLib.markup_escape_text ("(Mostrar mais...)", -1),
            style_class: 'journal-heading',
            x_align: St.Align.START,
            y_align: St.Align.START,
            can_focus: true });
        this._header.add(this._moreButton, {x_align: St.Align.START, y_align: St.Align.START, expand: false, x_fill: true, y_fill: true });
        this._moreButton.hide();
        this._moreButton.connect('clicked', Lang.bind(this, function() {
            this._toggleMore();
        }));

        this._results = [];
        let size = searchContent([this._subFilter], this._results, this._filter, this._page);

        this._pages = parseInt(size/MAX_RESULTS_PER_PAGE);
        this._expandable = this._results.length > ITEM_LIMIT;

        if(this._pages > 1)
        {
            var pageLabel = new HeadingItem("   Pagina: ");
            this._header.add_actor(pageLabel.actor, {x_align: St.Align.START, y_align: St.Align.START, expand: false, x_fill: true, y_fill: true });
            this._pageButtons = [];
            this._pageShowing = [0, 0];

            this._before = new St.Button({
                    label: GLib.markup_escape_text ("<<", -1),
                    style_class: 'journal-heading',
                    x_align: St.Align.START,
                    y_align: St.Align.START,
                    can_focus: true });
            this._next = new St.Button({
                    label: GLib.markup_escape_text (">>", -1),
                    style_class: 'journal-heading',
                    x_align: St.Align.START,
                    y_align: St.Align.START,
                    can_focus: true });
            this._before.hide();
            this._next.hide();
            this._before.connect('clicked', Lang.bind(this, this._beforeAction));
            this._next.connect('clicked', Lang.bind(this, this._nextAction));
            this._header.add(this._before, {x_align: St.Align.START, y_align: St.Align.START, expand: false, x_fill: true, y_fill: true });

            if(this._pages*MAX_RESULTS_PER_PAGE < size)
                this._pages++;

            var pageStyle = 'journal-actived';
            for( i=0 ; i < this._pages ; i++)
            {
                this._pageButtons.push(new St.Button({
                    label: GLib.markup_escape_text ((i+1) + " ", -1),
                    style_class: pageStyle,
                    x_align: St.Align.START,
                    y_align: St.Align.START,
                    can_focus: true }));
                this._pageButtons[i].connect('clicked', Lang.bind(this, this._pageAction, i));
                this._header.add(this._pageButtons[i], {x_align: St.Align.START, y_align: St.Align.START, expand: false, x_fill: true, y_fill: true });
                pageStyle = 'journal-heading';
                if(i > MAX_PAGES-1)
                {
                    this._pageButtons[i].hide();
                    this._next.show();
                    this._pageShowing = [0, MAX_PAGES-1];
                }
            }
            this._header.add(this._next, {x_align: St.Align.START, y_align: St.Align.START, expand: false, x_fill: true, y_fill: true });
        }

        this._appendEvents();
    },

    _nextAction: function()
    {
        if( this._pageShowing[1] + MAX_PAGES+1 <= this._pageButtons.length)
            this._pageShowing[1] += MAX_PAGES;
        else
        {
            this._pageShowing[1] = this._pageButtons.length;
            this._next.hide();
        }
        this._pageShowing[0] = this._pageShowing[1] - (MAX_PAGES - 1);
        if(this._pageShowing[0] == 0)
            this._before.hide();
        else
            this._before.show();

        for(var i=0 ; i < this._pageButtons.length ; i++)
        {
            if(i >= this._pageShowing[0] && i <= this._pageShowing[1])
                this._pageButtons[i].show();
            else
                this._pageButtons[i].hide();
        }
    },

    _beforeAction: function()
    {
        if( this._pageShowing[0] - MAX_PAGES > 0)
            this._pageShowing[0] -= MAX_PAGES;
        else
        {
            this._pageShowing[0] = 0;
            this._before.hide();
        }
        this._pageShowing[1] = this._pageShowing[0] + (MAX_PAGES-1);
        if(this._pageShowing[1]+1 < this._pageButtons.length)
            this._next.show();
        else
            this._next.hide();

        for(var i=0 ; i < this._pageButtons.length ; i++)
        {
            if(i >= this._pageShowing[0] && i <= this._pageShowing[1])
                this._pageButtons[i].show();
            else
                this._pageButtons[i].hide();
        }
    },

    _loadPage: function()
    {
        this._results = [];
        searchContent([this._subFilter], this._results, this._filter, this._page);
        this._expandable = this._results.length > ITEM_LIMIT;

        this._appendEvents();
    },

    _appendEvents: function() {
        this.actor.hide();
        var inserted = 0;
        this._inserted_items = [];
        //log ("got " + events.length + " events");
        if (!this._expandable)
            this._item_limit = 999;
        for (let i = 0; i < this._results.length; i++) {
            let result = this._results[i];
            let uri = result.path;
            uri = GLib.uri_unescape_string(uri, '');

            if (GLib.file_test(uri, GLib.FileTest.EXISTS))
            {
                let item = new EventItem (result, this._multi_select, this);

                this._inserted_items.push(result);
                inserted = inserted +1;
                if (inserted <= this._item_limit) {
                    this.appendItem (item);
                    this.appendHSpace ();
                }
            }
        }

        if (inserted > 0)
        {
            this.actor.show();
        }
        
        //Util.spawn(['imprime_debug', this._expandable.toString()]); // DEBUG: #################################
        if (this._expandable) {
            this._moreButton.show();

            if (!(this._label in SubjCategories) && inserted > this._item_limit){
                this._moreButton.set_label("(Mostrar mais " + (this._inserted_items.length - this._item_limit) + " conte\xFAdo(s)...)");
            }
            else if (this._label in SubjCategories) {
                this._moreButton.set_label("(Mostrar mais...)");
                this._moreButton.connect('clicked', Lang.bind(this, function() {
                    [arg, func] = SubjCategories[this._label];
                func._selectCategory(arg);
                }));
            }
        }
    },

    _pageAction: function (object, pspec, page)
    {
        for(var j=0 ; j<this._pageButtons.length ; j++)
        {
            if(j == page)
                this._pageButtons[j].style_class = 'journal-actived';
            else
                this._pageButtons[j].style_class = 'journal-heading';
        }
        this._page = page;
        this._expanded = false;
        this._item_limit = ITEM_LIMIT;
        var widgets = this._container.get_children();
        this._items = [];
        for (var i = 0; i < widgets.length; i++)
        {
            this._container.remove_actor(widgets[i]);
            widgets[i].destroy();
        }
        this._loadPage();
        if (this._expandable) {
            let more_items = this._inserted_items.length - this._item_limit;
            this._moreButton.set_label("(Mostrar mais " + (more_items) + " conte\xFAdo(s)...)");
        } else {
            this._moreButton.hide();
        }
    },
    
    _toggleMore: function () {
        this._expanded = !this._expanded;
        if (this._expanded) {
            this._moreButton.set_label("(Esconder alguns conte\xFAdos)");
            this._item_limit = 999;
        }
        else {
            this._item_limit = ITEM_LIMIT;
            this._moreButton.set_label("(+ " + (this._inserted_items.length - this._item_limit) + " items...)")
        }
        
        var widgets = this._container.get_children();
        this._items = []
        for (var i = 0; i < widgets.length; i++)
        {
            this._container.remove_actor(widgets[i]);
            widgets[i].destroy();
        }
        
        this._appendEvents();
    },
    
    _styleChanged: function () {
        for (var key in this._containers)
        {
            let node = this._containers[key].get_theme_node ();

            this._itemSpacing = node.get_length ("item-spacing");
            this._rowSpacing = node.get_length ("row-spacing");

            this._containers[key].queue_relayout ();
        }
    },
    
    _allocate: function (actor, box, flags) {
        let width = box.x2 - box.x1;
        this._computeLayout (width, true, flags);
    },
    
    _getPreferredWidth: function (actor, forHeight, alloc) {
        alloc.min_size = 128; // FIXME: get the icon size from CSS
        alloc.natural_size = (48 + this._itemSpacing) * 4 - this._itemSpacing; // four horizontal icons and the spacing between them
    },

    _getPreferredHeight: function (actor, forWidth, alloc) {
        let height = this._computeLayout (forWidth, true, null);
        alloc.min_size = height;
        alloc.natural_size = height;
    },

    _computeLayout: function (available_width, do_allocation, allocate_flags) {
        let layout_state = { newline_goal_column: 0,
                             x: 0,
                             y: 0,
                             row_height : 0,
                             layout_width: available_width };

        let newline = Lang.bind (this, function () {
            layout_state.x = layout_state.newline_goal_column;
            layout_state.y += layout_state.row_height + this._rowSpacing;
            layout_state.row_height = 0;
        });

        for (let i = 0; i < this._items.length; i++) {
            let item = this._items[i];
            let item_layout = { width: 0, height: 0 };

            if (item.type == "item") {
                if (!item.child)
                    throw new Error ("internal error - item.child must not be null");

                item_layout.width = item.child.actor.get_preferred_width (-1)[1]; // [0] is minimum width; [1] is natural width
                item_layout.height = item.child.actor.get_preferred_height (item_layout.width)[1];
            } else if (item.type == "newline") {
                newline ();
                continue;
            } else if (item.type == "hspace") {
                item_layout.width = this._itemSpacing;
            }

            if (layout_state.x + item_layout.width > layout_state.layout_width) {
                newline ();

                if (item.type == "hspace")
                    continue;
            }

            let box = new Clutter.ActorBox ();
            box.x1 = layout_state.x;
            box.y1 = layout_state.y;
            box.x2 = box.x1 + item_layout.width;
            box.y2 = box.y1 + item_layout.height;
            
            if (item.type == "item" && do_allocation && allocate_flags != null)
            {
                item.child.actor.allocate (box, allocate_flags);
            }

            layout_state.x += item_layout.width;
            if (item_layout.height > layout_state.row_height)
                layout_state.row_height = item_layout.height;
        }

        return layout_state.y + layout_state.row_height;
    },
    
    // We only expect items to have an item.actor field, which is a ClutterActor
    appendItem: function (item) {
        if (!item)
            throw new Error ("item must not be null");
        if (!item.actor)
            throw new Error ("Item must already contain an actor when added to the JournalLayout");
        let i = { type: "item",
                  child: item };
        this._items.push (i);
        this._container.add_actor (item.actor);
    },

    appendNewline: function () {
        let i = { type: "newline" }
        this._items.push (i);
    },

    appendHSpace: function () {
        let i = { type: "hspace" };
        this._items.push (i);
    },
    
    removeItem: function (item) {
        for (var i = 0; i < this._items.length; i++) {
            if (this._items[i].type == 'item' && this._items[i].child == item)
            {
                this._items.splice(i, i+1);
                log("-------------------------------------"); 
                break;
            }
        } 
    }
    
}

function MultiSelect ()
{
        this._init ();
}

MultiSelect.prototype = {
        _init: function () {
                this._elements = [];
                this._multi_select = false;
        },

        select: function (source, item) {
                if (this._elements.length == 0)
                        this._multi_select = true;
                else
                        this._multi_select = false;

                if (this.isSelected (item) || this._multi_select) {
                        let e = { source : source,
                                          item: item,
                                          selected: true };
                        source.add_style_class_name('journal-item-selection');
                        this._elements.push(e);
                } else {
                        this.unselect (source, item);
                }
        },

        unselect: function (source, item) {
                source.remove_style_class_name('journal-item-selection');        
                for (let i = 0; i < this._elements.length; i++) {
                        if (this._elements[i].source == source) {
                                this._elements.splice(i, 1);
                                break;
                        }
                }
        },

        isSelected: function (item) {
                for (let i = 0; i < this._elements.length; i++) {
                        let e = this._elements[i];
                        if ((e.item == item) && e.selected) {
                                return false;
                        }
                }
                return true;
        },

        destroy: function() {
          let elements = this._elements;
          for (let i = 0; i < elements.length; i++) {
                  let e = elements[i];
                  e.source.remove_style_class_name('journal-item-selection');        
          }
          this._elements = [];
        },

        querySelections: function () {
                return this._elements;
        }
}

function EducLayout()
{
    this._init();
}

EducLayout.prototype =
{
    _init: function()
    {
        this._items = [];

        this._itemSpacing = 0;
        this._rowSpacing = 0;

        this._box = new St.BoxLayout({name: 'searchResultsContent', vertical: true});
        this.actor = this._box;

        this._containers = {};
    },


    _setFilterViews: function (format, category)
    {
        this.clear()
        //var end = new Date().getTime();
        //let template = category.event_template;
        let offset = category.time_range;
        let sorting = category.sorting;

        let multi_select = new MultiSelect ();

        let uri_map = {}

        if (format == false)
        {
            this._containers["Lesson"] = new SubEduc (LESSON, sorting, multi_select, LESSON, category.filter, true);
            this._box.add_actor (this._containers["Lesson"].actor, { y_align: St.Align.START, expand: true, x_fill: true, y_fill: true });

            this._containers["Text"] = new SubEduc (TEXT, sorting, multi_select, TEXT, category.filter, true);
            this._box.add_actor (this._containers["Text"].actor, { y_align: St.Align.START, expand: true, x_fill: true, y_fill: true });

            this._containers["Video"] = new SubEduc (VIDEO, sorting, multi_select, VIDEO, category.filter, true);
            this._box.add_actor (this._containers["Video"].actor, { y_align: St.Align.START, expand: true, x_fill: true, y_fill: true });

            this._containers["Sound"] = new SubEduc (SOUND, sorting, multi_select, SOUND, category.filter, true);
            this._box.add_actor (this._containers["Sound"].actor, { y_align: St.Align.START, expand: true, x_fill: true, y_fill: true });

            this._containers["Picture"] = new SubEduc (PICTURE, sorting, multi_select, PICTURE, category.filter, true);
            this._box.add_actor (this._containers["Picture"].actor, { y_align: St.Align.START, expand: true, x_fill: true, y_fill: true });
        }
        else
        {
            this._containers["Contents"] = new SubEduc (category.subFilter, sorting, multi_select, category.subFilter, category.filter, true);
            this._box.add_actor (this._containers["Contents"].actor, { y_align: St.Align.START, expand: true, x_fill: true, y_fill: true });
        }

    },
    clear: function ()
    {
        this._items = [];
        for (var key in this._containers)
        {
            this._containers[key].actor.destroy_all_children();
            this._containers[key].actor.destroy();
            this.actor.destroy_all_children();
        }
    }
}

/* ------------------------------------------------------------------------------------------------- */

function EducDisplay()
{
    this._init();
}

EducDisplay.prototype =
{
    _init: function ()
    {
        this.box = new St.BoxLayout({style_class: 'all-app' });
        this._scroll_view = new St.ScrollView({ x_fill: false,
            y_fill: false,
            style_class: 'vfade' });

        this._scroll_view.set_policy (Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
        this._scroll_view.connect ("notify::mapped", Lang.bind (this, this._scrollViewMapCb));

        this._layout = new EducLayout ();

        this._filters = new St.BoxLayout({ vertical: true, reactive: true });
        this._scroll_view.add_actor(this._layout.actor, { expand: true, y_fill: true,y_align: St.Align.START, x_fill: true });

        this._categoryScroll = new St.ScrollView({ x_fill: false,
            y_fill: false,
            style_class: 'vfade' });
        this._categoryScroll.add_actor(this._filters);

        this.box.add(this._scroll_view, { expand: true, x_fill: true, y_fill: true });
        this.box.add_actor(this._categoryScroll, { expand: false, y_fill: false, y_align: St.Align.START  });

        this.actor = this.box;
        this._sections = [];
        this._setFilters();
        this._selectCategory(1);
        //this._filters.connect('scroll-event', Lang.bind(this, this._scrollFilter))
    },

    _scrollFilter: function(actor, event)
    {
        let direction = event.get_scroll_direction();
        if (direction == Clutter.ScrollDirection.UP)
            this._selectCategory(Math.max(this._currentCategory - 1, -1))
        else if (direction == Clutter.ScrollDirection.DOWN)
            this._selectCategory(Math.min(this._currentCategory + 1, this._sections.length - 1));
    },

    _selectCategory: function(num)
    {
        this._currentCategory = num;

        for (let i = 0; i < this._sections.length; i++)
        {
            if (i == num)
                this._sections[i].add_style_pseudo_class('selected');
            else
                this._sections[i].remove_style_pseudo_class('selected');
        }


        this._layout._setFilterViews((num > 2), this._categories[num]);
    },

    _addCategory: function (category, linkable)
    {
        let button = new St.Button({ label: GLib.markup_escape_text (category.title, -1),
            style_class: 'app-filter',
            x_align: St.Align.START,
            can_focus: true });
        this._filters.add(button, { expand: false, x_fill: true, y_fill: false });

        this._sections[this._counter] = button;

        var x = this._counter;
        button.connect('clicked', Lang.bind(this, function() {
            this._selectCategory(x);
        }));
        this._categories.push(category);
        this._counter = this._counter + 1;

        if (linkable == true)
            SubjCategories[category.title] = [x, this];
    },

    _scrollViewMapCb: function (actor)
    {
        if (this._scroll_view.mapped)
            this._reload ();
    },

    _reload: function ()
    {
        this._selectCategory(this._currentCategory)
    },

    _setFilters: function ()
    {
        this._counter = 0;
        this._categories = [];
        this._addCategory(new PublicDomain());
        this._addCategory(new SchoolTV());
        this._addCategory(new TeacherPortal());

        var space = new St.Label ({ text: "",
                                     style_class: 'app-filter' });
        this._filters.add(space, { expand: false, x_fill: true, y_fill: false });

        this._addCategory(new LessonFormat());
        this._addCategory(new TextFormat());
        this._addCategory(new VideoFormat());
        this._addCategory(new SoundFormat());
        this._addCategory(new PictureFormat());

        var space = new St.Label ({ text: "",
            style_class: 'app-filter' });
        this._filters.add(space, { expand: false, x_fill: true, y_fill: false });

        //this._addCategory(new DocumentsCategory(), true);
    },
}

/* ------------------------------------------------------------------------------------------------- */

function CategoryInterface(title)
{
    this._init(title);
}

CategoryInterface.prototype =
{
    _init: function(title)
    {
        this.title = title;
        this.func = null;
        this.subCategories = [];
        this.event_template = null;
        this.time_range = null;
        this.sorting = 2;
        this.filter = 'none';
        this.subFilter = -1;
    },
}

function PublicDomain()
{
    this._init();
}

PublicDomain. prototype =
{
    __proto__: CategoryInterface.prototype,
    _init: function()
    {
        CategoryInterface.prototype._init.call(this, _("Dom\xEDnio P\xFAblico"));
        this.filter = 'dp';
    }
}


function SchoolTV()
{
    this._init();
}

SchoolTV. prototype =
{
    __proto__: CategoryInterface.prototype,
    _init: function()
    {
        CategoryInterface.prototype._init.call(this, _("TV Escola"));
        this.filter = 'tve';
    }
}

function TeacherPortal()
{
    this._init();
}

TeacherPortal. prototype =
{
    __proto__: CategoryInterface.prototype,
    _init: function()
    {
        CategoryInterface.prototype._init.call(this, _("Portal do Professor"));
        this.filter = 'pp';
    }
}

function TextFormat()
{
    this._init();
}

TextFormat. prototype =
{
    __proto__: CategoryInterface.prototype,
    _init: function()
    {
        CategoryInterface.prototype._init.call(this, _("Texto"));
        this.subFilter = TEXT;
    }
}

function SoundFormat()
{
    this._init();
}

SoundFormat. prototype =
{
    __proto__: CategoryInterface.prototype,
    _init: function()
    {
        CategoryInterface.prototype._init.call(this, _("Som"));
        this.subFilter = SOUND;
    }
}
function PictureFormat()
{
    this._init();
}

PictureFormat. prototype =
{
    __proto__: CategoryInterface.prototype,
    _init: function()
    {
        CategoryInterface.prototype._init.call(this, _("Imagem"));
        this.subFilter = PICTURE;
    }
}
function VideoFormat()
{
    this._init();
}

VideoFormat. prototype =
{
    __proto__: CategoryInterface.prototype,
    _init: function()
    {
        CategoryInterface.prototype._init.call(this, _("V\xEDdeo"));
        this.subFilter = VIDEO;
    }
}
function LessonFormat()
{
    this._init();
}

LessonFormat. prototype =
{
    __proto__: CategoryInterface.prototype,
    _init: function()
    {
        CategoryInterface.prototype._init.call(this, _("Aula"));
        this.subFilter = LESSON;
    }
}
/* ------------------------------------------------------------------------------------------------- */

function init()
{
    imports.gettext.bindtextdomain('gnome-shell-extensions', Config.LOCALEDIR);
    educSearch = new EducSearch('CONTE\xDADOS EDUCACIONAIS', false);
    educInstall = new EducSearch('INSTALAR CONTE\xDADOS EDUCACIONAIS RELACIONADOS', true);
};

function enable() {
    Main.overview.addSearchProvider(educSearch);
    Main.overview.addSearchProvider(educInstall);

    searchDisplay = new EducDisplay();
    Main.overview._viewSelector.addViewTab('eduContents', _("Buscar Conte\xFAdos Educacionais"), searchDisplay.actor, 'system-run');
    viewTab = Main.overview._viewSelector._tabs[Main.overview._viewSelector._tabs.length-1];
    tabIndex = Main.overview._viewSelector._tabs.length - 1;
};

function disable() {
    Main.overview.removeSearchProvider(educSearch);
    Main.overview.removeSearchProvider(educInstall);

    Main.overview._viewSelector._tabBox.remove_actor(viewTab.title);
    Main.overview._viewSelector._pageArea.remove_actor(viewTab.page);
    Main.overview._viewSelector._tabs.splice(tabIndex, tabIndex);
    searchDisplay.actor.destroy();
    searchDisplay = null;
};
